!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   sp_experiments_mod
!> @brief   experimental data
!! @authors Takaharu Mori (TM)
!
!  (c) Copyright 2016 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif
  
module sp_experiments_mod

  use molecules_str_mod
  use sp_restraints_str_mod
  use sp_experiments_str_mod
  use sp_enefunc_str_mod
  use sp_domain_str_mod
  use sp_boundary_str_mod
  use fileio_sit_mod
  use fileio_control_mod
  use messages_mod
  use mpi_parallel_mod
  use constants_mod
  use string_mod
  use timers_mod
#ifdef MPI
  use mpi
#endif

  implicit none
  private

  ! structures
  type, public :: s_exp_info
    logical                         :: emfit           = .false.
    character(MaxFilename)          :: emfit_target    = ''
    real(wp)                        :: emfit_sigma     = 2.5_wp
    real(wp)                        :: emfit_tolerance = 0.001_wp
    integer                         :: emfit_period    = 1
  end type s_exp_info

  integer,                     save :: emfit_icycle
  integer,                     save :: max_nxyz
  integer,                     save :: iaxis0, ig_median0, natom_domain0
  integer,                     save :: natom_domain00, natom_domain10, natom_domain01
  integer,                     save :: ig_max00(3), ig_min00(3)
  integer,                     save :: ig_max10(3), ig_min10(3)
  integer,                     save :: ig_max01(3), ig_min01(3)
  integer,                     save :: my_sorted_rank
  integer,                     save :: domain_index1
  integer,                     save :: ntimer_count
  logical,                     save :: timer_started
  logical,                     save :: skip_emfit
  real(wp),                    save :: partition_ratio
  real(wp),                    save :: corrcoeff_save
  real(wp),       allocatable, save :: timing          (:)
  integer,        allocatable, save :: num_atoms_local (:)
  integer,        allocatable, save :: domain_index    (:)
  logical,        allocatable, save :: emfit_atom      (:)
  logical,        allocatable, save :: emfit_atom_local(:)
  integer,        allocatable, save :: ig_sort         (:)
  integer,        allocatable, save :: ig_domain0      (:,:)
  integer,        allocatable, save :: ig_domain       (:,:)
  integer,        allocatable, save :: ig_domain_tmp   (:,:)
  integer,        allocatable, save :: ematom_l2g      (:,:)
  integer,        allocatable, save :: ig_lower        (:,:)
  integer,        allocatable, save :: ig_upper        (:,:)
  real(wp),       allocatable, save :: emfit_force_tmp (:,:)
  real(wp),       allocatable, save :: emfit_force     (:,:)
  real(wp),       allocatable, save :: coord_domain0   (:,:)
  real(wp),       allocatable, save :: coord_domain1   (:,:)
  real(wp),       allocatable, save :: coord_tmp       (:,:)
  real(wp),       allocatable, save :: force_domain01  (:,:)
  real(wp),       allocatable, save :: force_domain10  (:,:)
  real(wp),       allocatable, save :: erfa            (:,:)
  real(wp),       allocatable, save :: expa            (:,:)
  real(wp),       allocatable, save :: derfa_x(:,:), derfa_y(:,:), derfa_z(:,:)
  real(wp),       allocatable, save :: dexpa_x(:,:), dexpa_y(:,:), dexpa_z(:,:)
  real(wp),       allocatable, save :: dot_exp_drhodx  (:,:)
  real(wp),       allocatable, save :: dot_sim_drhodx  (:,:)
  real(wp),       allocatable, save :: dot_exp_drhodx_tmp(:,:)
  real(wp),       allocatable, save :: dot_sim_drhodx_tmp(:,:)
  type(s_experiments), target, save :: experiments

  ! subroutines
  public  :: show_ctrl_experiments
  public  :: read_ctrl_experiments
  public  :: setup_experiments
  private :: setup_experiments_emfit
  public  :: emfit_pre
  public  :: compute_energy_experimental_restraint
  private :: compute_energy_experimental_restraint_emfit
  private :: quicksort

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    show_ctrl_experiments
  !> @brief        show EXPERIMENTS section usage
  !! @authors      TM
  !! @param[in]    show_all : show all usage or not
  !! @param[in]    run_mode : run mode string : "md", "remd", "min", "rpath"
  !
  !======1=========2=========3=========4=========5=========6=========7=========8
  
  subroutine show_ctrl_experiments(show_all, run_mode)

    ! formal arguments
    logical,                 intent(in)    :: show_all
    character(*),            intent(in)    :: run_mode

    return

  end subroutine show_ctrl_experiments
  
  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Function      read_ctrl_experiments
  !> @brief        read EXPERIMENTS section in the control file
  !! @authors      TM
  !! @param[in]    handle   : unit number of control files
  !! @param[out]   exp_info : EXPERIMENTS section control parameters information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine read_ctrl_experiments(handle, exp_info) 

    ! parameters
    character(*),            parameter     :: Section = 'Experiments'

    ! formal arguments
    integer,                 intent(in)    :: handle
    type(s_exp_info),        intent(inout) :: exp_info 

    return

  end subroutine read_ctrl_experiments

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Function      setup_experiments
  !> @brief        setup experiments information
  !! @authors      TM
  !! @param[in]    exp_info    : EXPERIMENTS section control parameters
  !! @param[inout] experiments : experiments information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine setup_experiments(exp_info, molecule, restraints, enefunc)

    ! formal arguments
    type(s_exp_info),        intent(in)    :: exp_info
    type(s_molecule),        intent(in)    :: molecule
    type(s_restraints),      intent(in)    :: restraints
    type(s_enefunc),         intent(inout) :: enefunc

    ! local variables
    integer    :: i
    logical    :: do_emfit_res

    return

  end subroutine setup_experiments

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Function      setup_experiments_emfit
  !> @brief        setup experiments information
  !! @authors      TM
  !! @param[in]    exp_info    : EXPERIMENTS section control parameter
  !! @param[inout] experiments : experiments information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine setup_experiments_emfit(exp_info, molecule, enefunc)

    ! formal arguments
    type(s_exp_info),        intent(in)    :: exp_info
    type(s_molecule),        intent(in)    :: molecule
    type(s_enefunc),         intent(inout) :: enefunc

    ! local variables
    type(s_sit)             :: sit
    integer                 :: i, j, k, idx, n(3)
    real(wp)                :: pi, r, y

    return

  end subroutine setup_experiments_emfit

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    compute_energy_experimental_restraint
  !! @authors      TM
  !! @param[in]    coord   : coordinates of target systems
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine emfit_pre(domain, boundary, coord)

    ! formal arguments
    type(s_domain),  target, intent(in)    :: domain
    type(s_boundary),        intent(in)    :: boundary
    real(dp),                intent(in)    :: coord(:,:,:)

    ! local variables
    integer           :: i, j, k, n, ix, ifound
    integer           :: ncell, ncel(3), n_grid_cut
    integer           :: ig_max_tmp, ig_min_tmp
    integer           :: ig_domain0_max(3), ig_domain0_min(3)
    integer           :: length0(3), i00, i10, i01, domain_index0
    integer           :: work1, work2, max_natom_domain, alloc_size
    integer           :: irequest0, irequest1, irequest2, irequest3
    integer           :: irequest4, irequest5
    integer           :: idata0(1:6), idata1(1:6)
    integer           :: ig_max0(3), ig_min0(3), nmap(3)
    integer           :: max_timing_domain1, max_timing_domain2
    integer           :: partition_update_period
    real(wp)          :: csize(3), map_ori(3)
    real(wp)          :: xmin_domain, xmax_domain
    real(wp)          :: inv_dx, timer_tmp, work3
    logical           :: start_domain(3), end_domain(3)
    logical           :: do_allocate
    integer,  pointer :: id_l2g(:,:), natom_cell(:)
#ifdef MPI
    integer           :: istatus(mpi_status_size)
#endif

    return

  end subroutine emfit_pre

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    compute_energy_experimental_restraint
  !> @brief        calculate restraint energy from experimental data
  !! @authors      TM
  !! @param[in]    enefunc : potential energy functions information
  !! @param[in]    coord   : coordinates of target systems
  !! @param[in]    inum    : pointer for restraint function
  !! @param[in]    const   : force constants
  !! @param[in]    ref     : reference value
  !! @param[inout] force   : forces of target systems
  !! @param[inout] virial  : virial of target systems
  !! @param[out]   eexp    : restraint energy
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine compute_energy_experimental_restraint                         &
                            (calc_force, domain, boundary, enefunc, coord, &
                             force, virial, sum_eemfit, sum_emcorr)

    ! formal arguments
    logical,                 intent(in)    :: calc_force
    type(s_domain),  target, intent(in)    :: domain
    type(s_boundary),        intent(in)    :: boundary
    type(s_enefunc), target, intent(inout) :: enefunc
    real(dp),                intent(in)    :: coord(:,:,:)
    real(wp),                intent(inout) :: force(:,:,:,:)
    real(dp),                intent(inout) :: virial(3,3,nthread)
    real(dp),                intent(inout) :: sum_eemfit
    real(dp),                intent(inout) :: sum_emcorr

    ! local variables
    integer                  :: inum
    real(dp)                 :: emcorr, eemfit

    return

  end subroutine compute_energy_experimental_restraint

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    compute_energy_experimental_restraint_emfit
  !> @brief        calculate restraint energy from experimental data
  !! @authors      TM
  !! @param[in]    enefunc : potential energy functions information
  !! @param[in]    coord   : coordinates of target systems
  !! @param[in]    inum    : pointer for restraint function
  !! @param[in]    const   : force constants
  !! @param[in]    ref     : reference value
  !! @param[inout] force   : forces of target systems
  !! @param[inout] virial  : virial of target systems
  !! @param[out]   eexp    : restraint energy
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine compute_energy_experimental_restraint_emfit(inum, calc_force, &
                domain, boundary, enefunc, coord, force, virial, eemfit, emcorr)

    ! formal arguments
    integer,                 intent(in)    :: inum
    logical,                 intent(in)    :: calc_force
    type(s_domain),  target, intent(in)    :: domain
    type(s_boundary),        intent(in)    :: boundary
    type(s_enefunc), target, intent(inout) :: enefunc
    real(dp),                intent(in)    :: coord(:,:,:)
    real(wp),                intent(inout) :: force(:,:,:,:)
    real(dp),                intent(inout) :: virial(3,3,nthread)
    real(dp),                intent(inout) :: eemfit
    real(dp),                intent(inout) :: emcorr

    ! local variables
    integer           :: i, j, k, n, m, natom_all, group_id, ifound, ix
    integer           :: i1, j1, k1, ncell
    integer           :: nlen, ixx, icycle, ncycle
    integer           :: idomain, itmp1, itmp2
    integer           :: id, omp_get_thread_num
    integer           :: n_grid_cut
    real(wp)          :: yzfactor, zfactor, max_length
    real(wp)          :: drho_dx(3), before_allreduce(2), after_allreduce(2)
    real(wp)          :: dcc_dx, dcc_dy, dcc_dz
    real(wp)          :: norm_sim, dot_exp_sim
    real(wp)          :: f, g, d, coeff_rho, coeff_drho
    real(wp)          :: pi, vol, corrcoeff
    real(wp)          :: exp_tmp(3), sim_tmp(3)
    real(wp)          :: emfit_force_norm, weight, inv_dx
    real(wp)          :: fact1, fact2, fact3
    integer           :: i00, i01
    integer           :: natom_domain
    integer           :: irequest6, irequest7
    real(wp)          :: map_ori(3)
#ifdef MPI
    integer           :: istatus(mpi_status_size)
#endif
    integer,  pointer :: id_g2l(:,:)
    integer,  pointer :: id_l2g(:,:), natom_cell(:)
    real(wp), pointer :: norm_exp, sigma
    real(wp), pointer :: simulated_map(:,:,:), target_map(:,:,:)
    real(wp), pointer :: bound_x(:), bound_y(:), bound_z(:)

    return

  end subroutine compute_energy_experimental_restraint_emfit

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    quicksort
  !> @brief        sorting function
  !! @authors      NT
  !! @param[inout] a     : list of integers
  !! @param[in]    start : start index
  !! @param[in]    end   : end index
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  recursive subroutine quicksort(a, start, end)

    ! formal arguments
    integer,                 intent(inout) :: a(*)
    integer,                 intent(in)    :: start
    integer,                 intent(in)    :: end

    ! local variables
    integer                  :: i, j, t, x


    x = a((start + end) / 2)
    i = start
    j = end

    do
      do while (a(i) < x)
        i = i + 1
      end do

      do while (x < a(j))
        j = j - 1
      end do

      if (i >= j) exit

      t = a(i)
      a(i) = a(j)
      a(j) = t
      i = i + 1
      j = j - 1

    end do

    if (start < i - 1) &
      call quicksort(a, start, i - 1)
    if (j + 1 < end) &
      call quicksort(a, j + 1, end)

    return

  end subroutine quicksort

end module sp_experiments_mod
